@@ -26,11 +26,21 @@
 
 G_BEGIN_DECLS
 
+#define NAUTILUS_BURN_ISO_ERROR nautilus_burn_iso_error_quark ()
+
+GQuark nautilus_burn_iso_error_quark (void);
+
+typedef enum {
+	NAUTILUS_BURN_ISO_ERROR_INTERNAL,
+	NAUTILUS_BURN_ISO_ERROR_GENERAL
+} NautilusBurnIsoError;
+
 int      nautilus_burn_make_iso        (const char *filename,
 					const char *label,
 					gboolean    warn_low_space,
 					gboolean    use_joliet,
-					gboolean    debug);
+					gboolean    debug,
+					GError    **error);
 void     nautilus_burn_make_iso_cancel (void);
 gboolean nautilus_burn_verify_iso      (const char *filename,
 					char      **name,
