@@ -33,6 +33,7 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <glib/glist.h>
+#include <glib/gstdio.h>
 #include <libgnomevfs/gnome-vfs.h>
 
 #ifndef HAVE_MKDTEMP
@@ -193,10 +194,11 @@ virtual_dir_lookup (VirtualNode *dir, const char *filename)
 static VirtualNode *
 virtual_node_lookup (VirtualRoot *root, const char *path, VirtualNode **parent)
 {
-	gchar *copy, *next;
+	gchar *copy, *next, *copy_orig;
 	VirtualNode *node;
 
-	copy = g_strdup (path);
+	copy_orig = g_strdup (path);
+	copy = copy_orig;
 
 	if (parent != NULL) {
 		*parent = NULL;
@@ -234,7 +236,8 @@ virtual_node_lookup (VirtualRoot *root, const char *path, VirtualNode **parent)
 
 		copy = next;
 	}
-	
+	g_free (copy_orig);
+
 	return node;
 }
 
@@ -285,7 +288,7 @@ virtual_create (VirtualRoot *root, VirtualNode *dir,
 	} else {
 		template = g_build_filename (root->tempdir, "file.XXXXXX", NULL);
 
-		fd = mkstemp (template);
+		fd = g_mkstemp (template);
 
 		if (fd < 0) {
 			g_free (template);
@@ -505,7 +508,7 @@ create_dir_link (VirtualRoot *root,
 		
 		path = g_build_filename (target_path, dirent->d_name, NULL);
 
-		if (stat (path, &stat_buf) == 0) {
+		if (g_lstat (path, &stat_buf) == 0) {
 			if (S_ISDIR (stat_buf.st_mode)) {
 				create_dir_link (root, dir, dirent->d_name, path);
 			} else {
@@ -548,7 +551,7 @@ create_link (const char *rootname,
 		return GNOME_VFS_ERROR_FILE_EXISTS;
 	}
 
-	if (stat (target_path, &stat_buf) < 0) {
+	if (g_lstat (target_path, &stat_buf) < 0) {
 		g_free (basename);
 		return GNOME_VFS_ERROR_NOT_FOUND;
 	}
