@@ -42,6 +42,10 @@
 #include <fcntl.h>
 #include <math.h>
 
+#ifndef INVALID_HANDLE
+#define INVALID_HANDLE (GINT_TO_POINTER(-1))
+#endif
+
 #ifdef USE_HAL
 #include <libhal.h>
 #endif /* USE_HAL */
@@ -60,6 +64,14 @@
 
 #include "nautilus-burn-drive.h"
 
+/* For dvd_plus_rw_utils.cpp */
+int    get_dvd_r_rw_profile (const char *name);
+int    get_mmc_profile      (int fd);
+int    get_disc_size_cd     (int fd);
+gint64 get_disc_size_dvd    (int fd, int mmc_profile);
+int    get_read_write_speed (int fd, int *read_speed, int *write_speed);
+int    get_disc_status      (int fd, int *empty, int *is_rewritable, int *is_blank);
+
 #define CD_ROM_SPEED 176
 
 static struct {
@@ -83,6 +95,51 @@ struct NautilusBurnDrivePriv {
 	char                         *udi;
 };
 
+#ifdef __FreeBSD__
+
+#define get_ioctl_handle_fd(x) (((struct cam_device *)x)->fd)
+
+static gpointer
+open_ioctl_handle (const char *device)
+{
+	struct cam_device *cam;
+
+	cam = cam_open_device (device, O_RDWR);
+
+	return (cam ? (gpointer)cam : INVALID_HANDLE);
+}
+
+static void
+close_ioctl_handle (gpointer handle)
+{
+	cam_close_device ((struct cam_device *)handle);
+}
+
+#else
+
+#define get_ioctl_handle_fd(x) (GPOINTER_TO_INT(x))
+
+static gpointer
+open_ioctl_handle (const char *device)
+{
+	int fd;
+
+ 	if ((fd = open (device, O_RDWR | O_EXCL | O_NONBLOCK)) < 0
+ 	    && (fd = open (device, O_RDONLY | O_EXCL | O_NONBLOCK)) < 0) {
+		return INVALID_HANDLE;
+	}
+
+	return GINT_TO_POINTER (fd);
+}
+
+static void
+close_ioctl_handle (gpointer handle)
+{
+	close (GPOINTER_TO_INT (handle));
+}
+
+#endif
+
 #ifdef USE_HAL
 static LibHalContext *
 get_hal_context (void)
@@ -105,10 +162,56 @@ get_hal_context (void)
 }
 #endif /* USE_HAL */
 
+static int
+get_device_max_write_speed (char *device)
+{
+	gpointer ioctl_handle;
+	int      fd;
+	int      max_speed;
+	int      read_speed, write_speed;
+
+	max_speed = -1;
+
+	ioctl_handle = open_ioctl_handle (device);
+	if (ioctl_handle == INVALID_HANDLE) {
+		return -1;
+	}
+
+	fd = get_ioctl_handle_fd (ioctl_handle);
+	get_read_write_speed (fd, &read_speed, &write_speed);
+	close_ioctl_handle (ioctl_handle);
+	max_speed = (int)floor  (write_speed) / CD_ROM_SPEED;
+
+	return max_speed;
+}
 
 /* Utility functions, be careful to have a match with what's use in the
  * different bits of code */
 #if !defined (__linux__)
+
+static int
+get_device_max_read_speed (char *device)
+{
+	gpointer ioctl_handle;
+	int      fd;
+	int      max_speed;
+	int      read_speed, write_speed;
+
+	max_speed = -1;
+
+	ioctl_handle = open_ioctl_handle (device);
+	if (ioctl_handle == INVALID_HANDLE) {
+		return -1;
+	}
+
+	fd = get_ioctl_handle_fd (ioctl_handle);
+	get_read_write_speed (fd, &read_speed, &write_speed);
+	close_ioctl_handle (ioctl_handle);
+	max_speed = (int)floor (read_speed) / CD_ROM_SPEED;
+
+	return max_speed;
+}
+
 static char *
 cdrecord_get_stdout_for_id (char *id)
 {
@@ -149,7 +252,7 @@ get_cd_properties (char                  *device,
 		   int                   *max_wr_speed,
 		   NautilusBurnDriveType *type)
 {
-	char *stdout_data, *rd_speed, *wr_speed, *drive_cap;
+	char *stdout_data, *drive_cap;
 
 	*max_rd_speed = -1;
 	*max_wr_speed = -1;
@@ -215,15 +318,6 @@ add_whitelist (NautilusBurnDrive *drive)
 }
 #endif /* !__linux__ */
 
-/* For dvd_plus_rw_utils.cpp */
-int get_dvd_r_rw_profile (const char *name);
-int get_mmc_profile (int fd);
-int get_disc_size_cd (int fd);
-gint64 get_disc_size_dvd (int fd, int mmc_profile);
-int get_read_write_speed (int fd, int *read_speed, int *write_speed);
-int get_disc_status (int fd, int *empty, int *is_rewritable, int *is_blank);
-
-
 static void
 add_dvd_plus (NautilusBurnDrive *drive)
 {
@@ -276,8 +370,9 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 						   gboolean   *has_data,
 						   gboolean   *has_audio)
 {
-	int fd;
-	int mmc_profile;
+	gpointer ioctl_handle;
+	int      fd;
+	int      mmc_profile;
 
 	g_return_val_if_fail (device != NULL, NAUTILUS_BURN_MEDIA_TYPE_ERROR);
 
@@ -286,14 +381,15 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 	if (has_data) *has_data = FALSE;
 	if (has_audio) *has_audio = FALSE;
 
-	if ((fd = open (device, O_RDWR | O_EXCL | O_NONBLOCK)) < 0
-	    && (fd = open (device, O_RDONLY | O_EXCL | O_NONBLOCK)) < 0) {
+	ioctl_handle = open_ioctl_handle (device);
+	if (ioctl_handle == INVALID_HANDLE) {
 		if (errno == EBUSY) {
 			return NAUTILUS_BURN_MEDIA_TYPE_BUSY;
 		}
 		return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 	}
 
+	fd = get_ioctl_handle_fd (ioctl_handle);
 	mmc_profile = get_mmc_profile (fd);
 
 	/* Couldn't get the data about the media */
@@ -303,12 +399,12 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 		opened = nautilus_burn_drive_door_open (mmc_profile, fd);
 
 		if (opened != FALSE) {
-			close (fd);
+			close_ioctl_handle (ioctl_handle);
 			return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 		} else {
 			int blank, rewrite, empty;
 			if (get_disc_status (fd, &empty, &rewrite, &blank) == 0) {
-				close (fd);
+				close_ioctl_handle (ioctl_handle);
 
 				if (is_rewritable)
 					*is_rewritable = rewrite;
@@ -325,12 +421,14 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 				else
 					return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
 			}
-			close (fd);
+
+			close_ioctl_handle (ioctl_handle);
+
 			return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
 		}
 	}
 
-	close (fd);
+	close_ioctl_handle (ioctl_handle);
 
 	if (is_blank)
 		*is_blank = mmc_profile & 0x10000;
@@ -565,23 +663,25 @@ nautilus_burn_drive_get_media_type_and_rewritable (NautilusBurnDrive *drive,
 gint64
 nautilus_burn_drive_get_media_size_from_path (const char *device)
 {
-	int    fd;
-	int    secs;
-	int    mmc_profile;
-	gint64 size;
+	gpointer ioctl_handle;
+	int      fd;
+	int      secs;
+	int      mmc_profile;
+	gint64   size;
 
 	g_return_val_if_fail (device != NULL, NAUTILUS_BURN_MEDIA_SIZE_UNKNOWN);
 
 	secs = 0;
 
-	if ((fd = open (device, O_RDWR | O_EXCL | O_NONBLOCK)) < 0
-	    && (fd = open (device, O_RDONLY | O_EXCL | O_NONBLOCK)) < 0) {
+	ioctl_handle = open_ioctl_handle (device);
+	if (ioctl_handle == INVALID_HANDLE) {
 		if (errno == EBUSY) {
 			return NAUTILUS_BURN_MEDIA_SIZE_BUSY;
 		}
 		return NAUTILUS_BURN_MEDIA_SIZE_UNKNOWN;
 	}
 
+	fd = get_ioctl_handle_fd (ioctl_handle);
 	mmc_profile = get_mmc_profile (fd);
 
 	/* See nautilus_burn_drive_get_media_type_from_path for details */
@@ -603,7 +703,7 @@ nautilus_burn_drive_get_media_size_from_path (const char *device)
 		size = NAUTILUS_BURN_MEDIA_SIZE_NA;
 	}
 
-	close (fd);
+	close_ioctl_handle (ioctl_handle);
 
 	return size;
 }
@@ -895,12 +995,6 @@ hal_scan (gboolean recorder_only)
 }
 #endif /* USE_HAL */
 
-#if defined(__linux__) || defined(__FreeBSD__)
-
-
-
-#endif /* __linux__ || __FreeBSD__ */
-
 #if defined (__linux__)
 
 static char **
@@ -1068,7 +1162,7 @@ get_cd_scsi_id (const char *dev,
 	g_free (devfile);
 
 	/* Avoid problems with Valgrind */
-	memset (&m_idlun, 1, sizeof (m_idlun));
+	memset (&m_idlun, 0, sizeof (m_idlun));
 	*bus = *id = *lun = -1;
 
 	if (fd < 0) {
@@ -1113,50 +1207,6 @@ lookup_scsi_unit (int               bus,
 	return NULL;
 }
 
-#if !defined(__linux)
-static int
-get_device_max_read_speed (char *device)
-{
-	int fd;
-	int max_speed;
-	int read_speed, write_speed;
-
-	max_speed = -1;
-
-	if ((fd = open (device, O_RDWR | O_EXCL | O_NONBLOCK)) < 0
-	    && (fd = open (device, O_RDONLY | O_EXCL | O_NONBLOCK)) < 0) {
-		return -1;
-	}
-
-	get_read_write_speed (fd, &read_speed, &write_speed);
-	close (fd);
-	max_speed = (int)floor  (read_speed) / CD_ROM_SPEED;
-
-	return max_speed;
-}
-#endif
-
-static int
-get_device_max_write_speed (char *device)
-{
-	int fd;
-	int max_speed;
-	int read_speed, write_speed;
-
-	max_speed = -1;
-
-	if ((fd = open (device, O_RDWR | O_EXCL | O_NONBLOCK)) < 0
-	    && (fd = open (device, O_RDONLY | O_EXCL | O_NONBLOCK)) < 0) {
-		return -1;
-	}
-
-	get_read_write_speed (fd, &read_speed, &write_speed);
-	close (fd);
-	max_speed = (int)floor  (write_speed) / CD_ROM_SPEED;
-
-	return max_speed;
-}
-
 static char *
 get_scsi_cd_name (int               bus,
 		  int               id,
