@@ -78,7 +78,7 @@ ncb_hig_dialog (GtkMessageType type,
 					GTK_BUTTONS_NONE,
 					title);
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog), reason);
-	gtk_window_set_title (GTK_WINDOW (error_dialog), title);
+	gtk_window_set_title (GTK_WINDOW (error_dialog), "");
 	gtk_window_set_icon_name (GTK_WINDOW (error_dialog), "gnome-dev-cdrom");
 
 	gtk_container_set_border_width (GTK_CONTAINER (error_dialog), 5);
@@ -359,24 +359,31 @@ insert_media_request_cb (NautilusBurnRecorder *cdrecorder,
 }
 
 static gboolean
-warn_data_loss_cb (NautilusBurnRecorder *recorder, gpointer data)
+ask_rewrite_disc (void)
 {
 	GtkWidget *dialog;
-	int res;
+	int        res;
 
 	dialog = ncb_hig_dialog (GTK_MESSAGE_WARNING,
-			_("The inserted disc was already written to."),
-			_("Do you want to insert another disc, or erase this one?"), NULL);
+				 _("The inserted disc was already written to."),
+				 _("Do you want to insert another disc, or erase this one?"), NULL);
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-			_("Insert another disc"), GTK_RESPONSE_NO,
-			_("Erase this disc"), GTK_RESPONSE_YES,
-			NULL);
+				_("Insert another disc"), GTK_RESPONSE_NO,
+				_("Erase this disc"), GTK_RESPONSE_YES,
+				NULL);
 	res = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
 	return (res == GTK_RESPONSE_YES);
 }
 
+static gboolean
+warn_data_loss_cb (NautilusBurnRecorder *recorder,
+		   gpointer              data)
+{
+	return !ask_rewrite_disc ();
+}
+
 static void
 details_clicked (GtkButton *button,
 		 gpointer   data)
@@ -613,7 +620,7 @@ media_type_matches (NautilusBurnMediaType type,
 		    gboolean              is_blank,
 		    gboolean              was_unmounted)
 {
-	if (was_unmounted != FALSE) {
+	if (was_unmounted) {
 		switch (type) {
 		case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
 		case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
@@ -727,6 +734,11 @@ request_media (NautilusBurnDrive *drive,
 
 	}
 
+	if (!is_blank) {
+		/* don't ask user for confirmation of blank here, wait for warn-data-loss signal */
+		/*return ask_rewrite_disc ();*/
+	}
+
 	return TRUE;
 }
 
@@ -811,9 +823,18 @@ burn_cd (const NautilusBurnDrive *drive,
 
 	if (drive->type != NAUTILUS_BURN_DRIVE_TYPE_FILE) {
 		gboolean try_overburn;
+		gboolean is_locked;
+		gboolean got_media;
 
-		if (!request_media ((NautilusBurnDrive *)drive, burn_size, overburn, &try_overburn))
+		is_locked = nautilus_burn_drive_lock ((NautilusBurnDrive *)drive, _("Burning CD"), NULL);
+		got_media = request_media ((NautilusBurnDrive *)drive, burn_size, overburn, &try_overburn);
+		if (is_locked) {
+			nautilus_burn_drive_unlock ((NautilusBurnDrive *)drive);
+		} 
+
+		if (!got_media) {
 			return 1;
+		}
 
 		if (try_overburn)
 			flags |= NAUTILUS_BURN_RECORDER_WRITE_OVERBURN;
@@ -989,23 +1010,25 @@ refresh_dialog (GladeXML *xml)
 	g_object_set_data (G_OBJECT (item), "speed", GINT_TO_POINTER (0));
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	for (i = 1; i <= rec->max_speed_write; i++) {
-		name = g_strdup_printf ("%dx", i);
-		item = gtk_menu_item_new_with_label (name);
-		g_object_set_data (G_OBJECT (item), "speed", GINT_TO_POINTER (i));
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		g_free (name);
+	if (rec) {
+		for (i = 1; i <= rec->max_speed_write; i++) {
+			name = g_strdup_printf ("%dx", i);
+			item = gtk_menu_item_new_with_label (name);
+			g_object_set_data (G_OBJECT (item), "speed", GINT_TO_POINTER (i));
+			gtk_widget_show (item);
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+			g_free (name);
+		}
+
+		/* Disable speed if max speed < 1 */
+		gtk_widget_set_sensitive (option_menu, rec->max_speed_write > 0);
+		/* Disable eject for burn-to-iso */
+		eject_check = glade_xml_get_widget (xml, "eject_check");
+		gtk_widget_set_sensitive (eject_check, rec->type != NAUTILUS_BURN_DRIVE_TYPE_FILE);
 	}
+
 	/* Default to automatic */
 	gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), MIN (default_speed, default_speed));
-
-	/* Disable speed if max speed < 1 */
-	gtk_widget_set_sensitive (option_menu, rec->max_speed_write > 0);
-
-	/* Disable eject for burn-to-iso */
-	eject_check = glade_xml_get_widget (xml, "eject_check");
-	gtk_widget_set_sensitive (eject_check, rec->type != NAUTILUS_BURN_DRIVE_TYPE_FILE);
 }
 
 static void
@@ -1127,17 +1150,23 @@ static gboolean
 ask_cancel (void)
 {
 	GtkWidget *warning_dialog;
+	GtkWidget *image;
+	GtkWidget *button;
 	int        res;
 
 	warning_dialog = ncb_hig_dialog (GTK_MESSAGE_WARNING,
-					 _("Cancel write"),
-					 _("Are you sure you want to cancel the disc write operation?\n"
-					   "Some optical media drives may require that you restart the machine to get it working again."),
+					 _("Interrupt writing files to disc?"),
+					 _("Are you sure you want to interrupt the disc write operation? "
+					   "Some optical media drives may require that you restart the computer to get them working again."),
 					 nautilus_burn_progress_get_window ());
-	gtk_dialog_add_buttons (GTK_DIALOG (warning_dialog),
-				_("Continue"), GTK_RESPONSE_NO,
-				GTK_STOCK_CANCEL, GTK_RESPONSE_YES,
-				NULL);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_STOP, GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (image);
+	gtk_dialog_add_button (GTK_DIALOG (warning_dialog), _("Continue"), GTK_RESPONSE_NO);
+	button = gtk_dialog_add_button (GTK_DIALOG (warning_dialog), _("Interrupt"), GTK_RESPONSE_YES);
+	g_object_set (button, "image", image, NULL);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (warning_dialog), GTK_RESPONSE_NO);
 	res = gtk_dialog_run (GTK_DIALOG (warning_dialog));
 	gtk_widget_destroy (warning_dialog);
 
@@ -1382,14 +1411,14 @@ main (int argc, char *argv[])
 		}
 	} while (res == GTK_RESPONSE_HELP);
 
-	gtk_widget_hide (dialog);
-	nautilus_burn_progress_image_setup ();
-	gtk_widget_show (progress_window);
-	cancel_button = glade_xml_get_widget (xml, "cancel_button");
-	g_signal_connect (cancel_button, "clicked", (GCallback)cancel_clicked, NULL);
-	g_signal_connect (progress_window, "delete_event", (GCallback)handle_delete_event, NULL);
+	if (res == GTK_RESPONSE_OK) {
+		gtk_widget_hide (dialog);
+		nautilus_burn_progress_image_setup ();
+		gtk_widget_show (progress_window);
+		cancel_button = glade_xml_get_widget (xml, "cancel_button");
+		g_signal_connect (cancel_button, "clicked", (GCallback)cancel_clicked, NULL);
+		g_signal_connect (progress_window, "delete_event", (GCallback)handle_delete_event, NULL);
 
-	if (res == 0) {
 		rec = lookup_current_recorder (xml);
 
 		entry = glade_xml_get_widget (xml, "cdname_entry");
@@ -1431,9 +1460,10 @@ main (int argc, char *argv[])
 						    GNOME_VFS_XFER_RECURSIVE,
 						    NULL, NULL);
 		}
+
+		nautilus_burn_progress_image_cleanup ();
 	}
 
-	nautilus_burn_progress_image_cleanup ();
 	gtk_widget_destroy (dialog);
 
 	return 0;
